@@ -52,6 +52,10 @@ static void     gail_focus_notify_when_idle (GtkWidget            *widget);
 static void     gail_focus_tracker_init (void);
 static void     gail_focus_object_destroyed (gpointer data);
 static void     gail_focus_tracker (AtkObject *object);
+static void     gail_set_focus_widget (GtkWidget *focus_widget,
+                                       GtkWidget *widget); 
+static void     gail_set_focus_object (AtkObject *focus_obj,
+                                       AtkObject *obj);
 
 static GtkWidget* focus_widget = NULL;
 static GtkWidget* next_focus_widget = NULL;
@@ -116,7 +120,10 @@ gail_get_accessible_for_widget (GtkWidget *widget,
     {
       GtkWidget *other_widget = widget->parent;
       if (GTK_IS_COMBO (other_widget))
-        widget = other_widget;
+        {
+          gail_set_focus_widget (other_widget, widget);
+          widget = other_widget;
+        }
     } 
   else if (GTK_IS_NOTEBOOK (widget)) 
     {
@@ -153,7 +160,10 @@ gail_get_accessible_for_widget (GtkWidget *widget,
     {
       GtkWidget *other_widget = widget->parent;
       if (GTK_IS_COMBO_BOX (other_widget))
-        widget = other_widget;
+        {
+          gail_set_focus_widget (other_widget, widget);
+          widget = other_widget;
+        }
     }
   if (obj == NULL)
     {
@@ -595,6 +605,20 @@ gail_focus_notify_when_idle (GtkWidget *widget)
       next_focus_widget = widget;
       g_object_add_weak_pointer (G_OBJECT (next_focus_widget), vp_next_focus_widget);
     }
+  else
+    {
+      /*
+       * We are about to report focus as NULL so remove the weak pointer
+       * for the widget we were waiting to report focus on.
+       */ 
+      if (next_focus_widget)
+        {
+          void *vp_next_focus_widget = &next_focus_widget;
+          g_object_remove_weak_pointer (G_OBJECT (next_focus_widget), vp_next_focus_widget);
+          next_focus_widget = NULL;
+        }
+    }
+
   focus_notify_handler = g_idle_add (gail_focus_idle_handler, widget);
 }
 
@@ -640,7 +664,7 @@ gail_deactivate_watcher (GSignalInvocationHint *ihint,
 }
 
 static void
-gail_focus_tracker_init ()
+gail_focus_tracker_init (void)
 {
   static gboolean  emission_hooks_added = FALSE;
 
@@ -730,27 +754,7 @@ gail_focus_tracker (AtkObject *focus_object)
 
           if (parent)
             {
-              old_focus_object = g_object_get_qdata (G_OBJECT (parent), quark_focus_object);
-              if (old_focus_object != focus_object)
-                {
-                  if (old_focus_object)
-                    g_object_weak_unref (G_OBJECT (old_focus_object),
-                                         (GWeakNotify) gail_focus_object_destroyed,
-                                         parent);
-                  else
-                    /*
-                     * We call g_object_ref as if the parent is destroyed 
-                     * while the weak reference exists then destroying the 
-                     * child would cause gail_focus_object_destroyed to be 
-                     * called when parent is not a valid GObject.
-                     */
-                    g_object_ref (parent);
-
-                  g_object_weak_ref (G_OBJECT (focus_object),
-                                     (GWeakNotify) gail_focus_object_destroyed,
-                                     parent);
-                  g_object_set_qdata (G_OBJECT (parent), quark_focus_object, focus_object);
-                }
+              gail_set_focus_object (focus_object, parent);
             }
         }
       else
@@ -768,6 +772,47 @@ gail_focus_tracker (AtkObject *focus_object)
     }
 }
 
+static void 
+gail_set_focus_widget (GtkWidget *focus_widget,
+                       GtkWidget *widget)
+{
+  AtkObject *focus_obj;
+  AtkObject *obj;
+
+  focus_obj = gtk_widget_get_accessible (focus_widget);
+  obj = gtk_widget_get_accessible (widget);
+  gail_set_focus_object (focus_obj, obj);
+}
+
+static void 
+gail_set_focus_object (AtkObject *focus_obj,
+                       AtkObject *obj)
+{
+  AtkObject *old_focus_obj;
+
+  old_focus_obj = g_object_get_qdata (G_OBJECT (obj), quark_focus_object);
+  if (old_focus_obj != obj)
+    {
+      if (old_focus_obj)
+        g_object_weak_unref (G_OBJECT (old_focus_obj),
+                             (GWeakNotify) gail_focus_object_destroyed,
+                             obj);
+      else
+        /*
+         * We call g_object_ref as if obj is destroyed 
+         * while the weak reference exists then destroying the 
+         * focus_obj would cause gail_focus_object_destroyed to be 
+         * called when obj is not a valid GObject.
+         */
+        g_object_ref (obj);
+
+      g_object_weak_ref (G_OBJECT (focus_obj),
+                         (GWeakNotify) gail_focus_object_destroyed,
+                         obj);
+      g_object_set_qdata (G_OBJECT (obj), quark_focus_object, focus_obj);
+    }
+}
+
 /*
  *   These exported symbols are hooked by gnome-program
  * to provide automatic module initialization and shutdown.
