@@ -22,6 +22,7 @@
 #include <gdk/gdkkeysyms.h>
 #include "gailentry.h"
 #include "gailcombo.h"
+#include "gailcombobox.h"
 #include <libgail-util/gailmisc.h>
 
 static void       gail_entry_class_init            (GailEntryClass       *klass);
@@ -421,7 +422,8 @@ gail_entry_get_index_in_parent (AtkObject *accessible)
    * otherwise do the normal thing.
    */
   if (accessible->accessible_parent)
-    if (GAIL_IS_COMBO (accessible->accessible_parent))
+    if (GAIL_IS_COMBO (accessible->accessible_parent) ||
+        GAIL_IS_COMBO_BOX (accessible->accessible_parent))
       return 1;
 
   return ATK_OBJECT_CLASS (parent_class)->get_index_in_parent (accessible);
@@ -785,7 +787,7 @@ gail_entry_get_selection (AtkText *text,
   * selection_num is 0.
   */
   if (selection_num != 0)
-     return 0;
+     return NULL;
 
   entry = GTK_ENTRY (widget);
   gtk_editable_get_selection_bounds (GTK_EDITABLE (entry), start_pos, end_pos);
@@ -793,7 +795,7 @@ gail_entry_get_selection (AtkText *text,
   if (*start_pos != *end_pos)
      return gtk_editable_get_chars (GTK_EDITABLE (entry), *start_pos, *end_pos);
   else
-     return 0;
+     return NULL;
 }
 
 static gboolean
