@@ -232,7 +232,7 @@ gail_sub_menu_item_ref_selection (AtkSelection   *selection,
     return NULL;
 
   submenu = gtk_menu_item_get_submenu (GTK_MENU_ITEM (widget));
-  g_return_val_if_fail (GTK_IS_MENU_SHELL (submenu), FALSE);
+  g_return_val_if_fail (GTK_IS_MENU_SHELL (submenu), NULL);
   shell = GTK_MENU_SHELL (submenu);
   
   if (shell->active_menu_item != NULL)
@@ -368,7 +368,7 @@ menu_item_remove_gtk (GtkContainer *container,
   AtkObject *atk_parent;
   AtkObject *atk_child;
   GailContainer *gail_container;
-  AtkPropertyValues values = { 0, };
+  AtkPropertyValues values = { NULL };
   gint index;
 
   g_return_val_if_fail (GTK_IS_MENU (container), 1);
