@@ -94,6 +94,8 @@ static gboolean   gail_widget_focus_gtk          (GtkWidget     *widget,
                                                   GdkEventFocus *event);
 static gboolean   gail_widget_real_focus_gtk     (GtkWidget     *widget,
                                                   GdkEventFocus *event);
+static void       gail_widget_size_allocate_gtk  (GtkWidget     *widget,
+                                                  GtkAllocation *allocation);
 
 static void       gail_widget_focus_event        (AtkObject     *obj,
                                                   gboolean      focus_in);
@@ -192,7 +194,11 @@ gail_widget_real_initialize (AtkObject *obj,
   g_signal_connect (widget,
                     "notify",
                     G_CALLBACK (gail_widget_notify_gtk),
-                    widget);
+                    NULL);
+  g_signal_connect (widget,
+                    "size_allocate",
+                    G_CALLBACK (gail_widget_size_allocate_gtk),
+                    NULL);
   atk_component_add_focus_handler (ATK_COMPONENT (accessible),
                                    gail_widget_focus_event);
   /*
@@ -906,6 +912,24 @@ gail_widget_real_focus_gtk (GtkWidget     *widget,
   return FALSE;
 }
 
+static void
+gail_widget_size_allocate_gtk (GtkWidget     *widget,
+                               GtkAllocation *allocation)
+{
+  AtkObject* accessible;
+  AtkRectangle rect;
+
+  accessible = gtk_widget_get_accessible (widget);
+  if (ATK_IS_COMPONENT (accessible))
+    {
+      rect.x = allocation->x;
+      rect.y = allocation->y;
+      rect.width = allocation->width;
+      rect.height = allocation->height;
+      g_signal_emit_by_name (accessible, "bounds_changed", &rect);
+    }
+}
+
 /*
  * This function is the signal handler defined for map and unmap signals.
  */
